<?php

use App\Domain\Catalog\Tests\Factories\Categories\CategoryFactory;
use App\Domain\Catalog\Tests\Factories\Classifiers\BrandFactory;
use App\Domain\Catalog\Tests\Factories\Offers\OfferFactory;
use App\Domain\Catalog\Tests\Factories\Offers\StockFactory;
use App\Domain\Catalog\Tests\Factories\Products\ProductFactory;
use App\Domain\Customers\Tests\Factories\CustomerFactory;
use App\Domain\Customers\Tests\Factories\CustomerUserFactory;
use App\Domain\Logistic\Tests\DeliveryServices\Factories\DeliveryServiceFactory;
use App\Domain\Orders\Tests\Factories\Orders\DeliveryFactory;
use App\Domain\Orders\Tests\Factories\Orders\OrderFactory;
use App\Domain\Orders\Tests\Factories\Orders\OrderItemFactory;
use App\Domain\Orders\Tests\Factories\Orders\ShipmentFactory;
use App\Domain\Units\Tests\Factories\AdminUserFactory;
use App\Domain\Units\Tests\Factories\StoreFactory;
use App\Http\ApiV1\Modules\Orders\Tests\Orders\Factories\AddOrderItemsRequestFactory;
use App\Http\ApiV1\Modules\Orders\Tests\Orders\Factories\ChangeOrderItemQtyRequestFactory;
use App\Http\ApiV1\Modules\Orders\Tests\Orders\Factories\OrderRequestFactory;
use App\Http\ApiV1\Support\Tests\ApiV1ComponentTestCase;
use Ensi\LaravelTestFactories\FakerProvider;
use Ensi\LaravelTestFactories\PaginationFactory;
use Ensi\LaravelTestFactories\PromiseFactory;
use Ensi\OmsClient\Dto\SearchOrdersRequest;

use function Pest\Laravel\getJson;
use function Pest\Laravel\patchJson;
use function Pest\Laravel\postJson;
use function PHPUnit\Framework\assertEquals;

uses(ApiV1ComponentTestCase::class);
uses()->group('component', 'orders', 'orders.orders');

test('POST /api/v1/orders/orders:search 200', function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var ApiV1ComponentTestCase $this */
    $orderId = 1;

    $this->mockOmsOrdersApi()->allows([
        'searchOrders' => OrderFactory::new()->makeResponseSearch([['id' => $orderId]]),
    ]);

    postJson("/api/v1/orders/orders:search", ['pagination' => PaginationFactory::new()->makeRequestOffset()])
        ->assertStatus(200)
        ->assertJsonPath('data.0.id', $orderId);
})->with(FakerProvider::$optionalDataset);

test('POST /api/v1/orders/orders:search all includes 200', function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var ApiV1ComponentTestCase $this */
    $orderId = 1;
    $responsibleId = 1;
    $customerId = 1;
    $customerUserId = 1;
    $offerId = 1;
    $productId = 1;
    $categoryId = 1;
    $brandId = 1;
    $storeId = 1;
    $omsIncludes = [
        "files",
        "deliveries",
        "deliveries.shipments",
        "deliveries.shipments.orderItems",
    ];
    $stockQty = 1000.4;

    $orderItem = OrderItemFactory::new()->make(['offer_id' => $offerId]);
    $shipment = ShipmentFactory::new()->withOrderItem($orderItem)->make(['store_id' => $storeId]);
    $delivery = DeliveryFactory::new()->withShipment($shipment)->make();
    $ordersResponse = OrderFactory::new()
        ->withFile()
        ->withDelivery($delivery)
        ->makeResponseSearch([['id' => $orderId, 'responsible_id' => $responsibleId, 'customer_id' => $customerId]]);
    $this->mockOmsOrdersApi()
        ->shouldReceive('searchOrders')
        ->withArgs(function ($arg) use ($omsIncludes) {
            /** @var SearchOrdersRequest $arg */
            assertEquals($omsIncludes, $arg->getInclude());

            return true;
        })
        ->andReturn($ordersResponse);
    $this->mockAdminAuthUsersApi()->allows([
        'searchUsersAsync' => PromiseFactory::make(AdminUserFactory::new()->makeResponseSearch([['id' => $responsibleId]])),
    ]);
    $this->mockCustomersCustomersApi()->allows([
        'searchCustomersAsync' => PromiseFactory::make(CustomerFactory::new()->makeResponseSearch([['id' => $customerId, 'user_id' => $customerUserId]])),
    ]);
    $this->mockCustomersAuthUsersApi()->allows([
        'searchUsersAsync' => PromiseFactory::make(CustomerUserFactory::new()->makeResponseSearch([['id' => $customerUserId]])),
    ]);
    $this->mockOffersOffersApi()->allows([
        'searchOffersAsync' => PromiseFactory::make(OfferFactory::new()
            ->withStock(StockFactory::new()->make(['offer_id' => $offerId, 'qty' => $stockQty]))
            ->makeResponseSearch([['id' => $offerId, 'product_id' => $productId]])),
    ]);
    $category = CategoryFactory::new()->make(['id' => $categoryId]);
    $brand = BrandFactory::new()->make(['id' => $brandId]);
    $this->mockPimProductsApi()->allows([
        'searchProductsAsync' => PromiseFactory::make(ProductFactory::new()
            ->withImages()
            ->withCategories($category)
            ->withBrand($brand)
            ->makeResponseSearch([['id' => $productId]])),
    ]);
    $this->mockBusinessUnitsStoresApi()->allows([
        'searchStoresAsync' => PromiseFactory::make(StoreFactory::new()->makeResponseSearch([['id' => $storeId]])),
    ]);

    postJson("/api/v1/orders/orders:search", [
        "include" => array_merge($omsIncludes, [
            "responsible",
            "customer",
            "customer.user",
            "orderItems.product",
            "orderItems.product.images",
            "orderItems.product.categories",
            "orderItems.product.brand",
            "orderItems.stock",
            "deliveries.shipments.store",
        ]),
    ])
        ->assertStatus(200)
        ->assertJsonPath('data.0.id', $orderId)
        ->assertJsonCount(1, 'data.0.files')
        ->assertJsonCount(1, 'data.0.deliveries')
        ->assertJsonCount(1, 'data.0.deliveries.0.shipments')
        ->assertJsonCount(1, 'data.0.deliveries.0.shipments.0.order_items')
        ->assertJsonPath('data.0.customer.id', $customerId)
        ->assertJsonPath('data.0.customer.user.id', $customerUserId)
        ->assertJsonPath('data.0.customer.user.id', $customerUserId)
        ->assertJsonPath('data.0.responsible.id', $responsibleId)
        ->assertJsonPath('data.0.deliveries.0.shipments.0.order_items.0.product.id', $productId)
        ->assertJsonCount(1, 'data.0.deliveries.0.shipments.0.order_items.0.product.images')
        ->assertJsonPath('data.0.deliveries.0.shipments.0.order_items.0.product.categories.0.id', $categoryId)
        ->assertJsonPath('data.0.deliveries.0.shipments.0.order_items.0.product.brand.id', $brandId)
        ->assertJsonPath('data.0.deliveries.0.shipments.0.store.id', $storeId)
        ->assertJsonPath('data.0.deliveries.0.shipments.0.order_items.0.stock.qty', $stockQty);
})->with(FakerProvider::$optionalDataset);

test('POST /api/v1/orders/orders/{id} 200', function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var ApiV1ComponentTestCase $this */
    $orderId = 1;

    $this->mockOmsOrdersApi()->allows([
        'getOrder' => OrderFactory::new()->makeResponse(['id' => $orderId]),
    ]);

    getJson("/api/v1/orders/orders/{$orderId}")
        ->assertStatus(200)
        ->assertJsonPath('data.id', $orderId);
})->with(FakerProvider::$optionalDataset);

test('PATCH /api/v1/orders/orders/{id} 200', function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var ApiV1ComponentTestCase $this */
    $orderId = 1;
    $pathOrderData = OrderRequestFactory::new()->makePathOrder();

    $this->mockOmsOrdersApi()->allows([
        'patchOrder' => OrderFactory::new()->makeResponse(array_merge(['id' => $orderId], $pathOrderData)),
    ]);

    // todo поле responsible_id скрыто в рамках задачи #90291
    patchJson("/api/v1/orders/orders/{$orderId}", $pathOrderData)
        ->assertStatus(200)
        ->assertJsonPath('data.id', $orderId)
//        ->assertJsonPath('data.responsible_id', $pathOrderData['responsible_id'])
        ->assertJsonPath('data.status', $pathOrderData['status'])
        ->assertJsonPath('data.client_comment', $pathOrderData['client_comment'])
        ->assertJsonPath('data.receiver_name', $pathOrderData['receiver_name'])
        ->assertJsonPath('data.receiver_phone', $pathOrderData['receiver_phone'])
        ->assertJsonPath('data.receiver_email', $pathOrderData['receiver_email'])
        ->assertJsonPath('data.is_problem', $pathOrderData['is_problem'])
        ->assertJsonPath('data.problem_comment', $pathOrderData['problem_comment']);
})->with(FakerProvider::$optionalDataset);

test('POST /api/v1/orders/orders/{id}:change-payment-method 200', function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var ApiV1ComponentTestCase $this */
    $paymentSystem = 4;

    $this->mockOmsOrdersApi()->allows([
        'changeOrderPaymentSystem' => OrderFactory::new()->makeResponse(['payment_system' => $paymentSystem]),
    ]);

    postJson("/api/v1/orders/orders/1:change-payment-method", ['payment_system' => $paymentSystem])
        ->assertStatus(200)
        ->assertJsonPath('data.payment_system', $paymentSystem);
})->with(FakerProvider::$optionalDataset);

test('POST /api/v1/orders/orders/{id}:change-delivery 200', function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var ApiV1ComponentTestCase $this */
    $orderDeliveryData = OrderRequestFactory::new()->makeChangeDelivery();

    $this->mockOmsOrdersApi()->allows([
        'changeOrderDelivery' => OrderFactory::new()->makeResponse($orderDeliveryData),
    ]);

    // todo поля скрыты в рамках задачи #90291
    postJson("/api/v1/orders/orders/1:change-delivery", $orderDeliveryData)
        ->assertStatus(200)
//        ->assertJsonPath('data.delivery_service', $orderDeliveryData['delivery_service'])
        ->assertJsonPath('data.delivery_method', $orderDeliveryData['delivery_method'])
//        ->assertJsonPath('data.delivery_tariff_id', $orderDeliveryData['delivery_tariff_id'])
//        ->assertJsonPath('data.delivery_point_id', $orderDeliveryData['delivery_point_id'])
//        ->assertJsonPath('data.delivery_price', $orderDeliveryData['delivery_price'])
//        ->assertJsonPath('data.delivery_cost', $orderDeliveryData['delivery_cost'])
        ->assertJsonPath('data.delivery_comment', $orderDeliveryData['delivery_comment']);
})->with(FakerProvider::$optionalDataset);

test('POST /api/v1/orders/orders/{id}:add-order-items 200', function (?bool $always) {
    /** @var ApiV1ComponentTestCase $this */
    FakerProvider::$optionalAlways = $always;

    $request = AddOrderItemsRequestFactory::new()->make();
    $this->mockOmsOrderItemsApi()->allows([
        'addOrderItems' => OrderFactory::new()->makeResponse([]),
    ]);

    postJson("/api/v1/orders/orders/1:add-order-items", $request)
        ->assertStatus(200);
})->with(FakerProvider::$optionalDataset);

test('POST /api/v1/orders/orders/{id}:change-order-items-qty 200', function (?bool $always) {
    /** @var ApiV1ComponentTestCase $this */
    FakerProvider::$optionalAlways = $always;

    $request = ChangeOrderItemQtyRequestFactory::new()->make();
    $this->mockOmsOrderItemsApi()->allows([
        'changeOrderItemQty' => OrderItemFactory::new()->makeResponseChange(),
    ]);

    postJson("/api/v1/orders/orders/1:change-order-items-qty", $request)
        ->assertStatus(200);
})->with(FakerProvider::$optionalDataset);

function mockMeta(ApiV1ComponentTestCase $testCase)
{
    $testCase->mockLogisticDeliveryServicesApi()->allows([
        'searchDeliveryServicesAsync' => PromiseFactory::make(DeliveryServiceFactory::new()->makeResponseSearch()),
    ]);
}
test('POST /api/v1/orders/orders:meta 200', function () {
    mockMeta($this);
    getJson("/api/v1/orders/orders:meta")->assertStatus(200);
});

test('POST /api/v1/orders/orders:meta all fields 200', function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var ApiV1ComponentTestCase $this */
    mockMeta($this);
    $request = getJson("/api/v1/orders/orders:meta");
    $metaFields = Arr::pluck($request->decodeResponseJson()['data']['fields'], 'code');
    $metaFields = array_diff($metaFields, ['delivery_address.address_string']);
    sort($metaFields);

    $this->mockOmsOrdersApi()->allows([
        'searchOrders' => OrderFactory::new()->makeResponseSearch(),
    ]);
    $request = postJson("/api/v1/orders/orders:search");
    $orderFields = array_keys($request->decodeResponseJson()['data'][0]);
    $orderFields = array_diff($orderFields, ['delivery_address']);

    sort($metaFields);
    sort($orderFields);
    assertEquals($metaFields, $orderFields);
})->with(FakerProvider::$optionalDataset);
